from pandac.PandaModules import Point3
from direct.actor.Actor import Actor
from showbase.DirectObject import DirectObject
from panda3d.core import PandaNode, NodePath, Camera, TextNode, ClockObject
from panda3d.core import CollisionHandlerQueue, \
    CollisionSphere, CollisionTraverser, CollisionNode, PandaNode, NodePath
from task.Task import TaskManager
from Grafo import Grafo
from direct.interval.FunctionInterval import Wait
from direct.interval.MetaInterval import Sequence

MAX = 150

class Agente :

    def __init__ (self) :
        self.startPos = Point3(0, 0, 0)
        self.modelo = Actor("../modelos/badralph",
                                 {"run": "../modelos/badralph-run", "walk": "../modelos/badralph-walk"})
        
        self.modelo.reparentTo(render)
        self.modelo.setScale(.3)
        self.modelo.setPos(self.startPos)

        self.floater = NodePath(PandaNode("floater"))
        self.floater.reparentTo(render)
        self.intervalColisao = None

        self.keyMap = {"left":0, "right":0, "forward":0}

        self.isMoving = False

        self.CollisionSphere = CollisionSphere(0, 0, 3, 1.1)
        self.CollisionNode = CollisionNode('personagemSphere')
        self.CollisionNode.addSolid(self.CollisionSphere)
        self.CollisionNodePath = self.modelo.attachNewNode(self.CollisionNode)
        self.CollisionHandler = CollisionHandlerQueue()
        
        self.grafo = Grafo()
        self.estadoInicial = 0
        self.estadoFinal = 149
        parent, self.dist = self.grafo.buscaLargura(self.estadoFinal)
        
     
    def subidaDeEncosta (self, x, y) :
        atual = int(abs(x) * 10 + abs(y))
        while True :
            vizinho = self.melhorVizinho(atual) 
            if self.avaliaPonto(vizinho) >= self.avaliaPonto(atual) :
                return atual
            self.criaMovimento(atual, vizinho)
            atual = vizinho
        
    def avaliaPonto (self, vizinho) :
        return self.dist[vizinho]
        
    def melhorVizinho (self, ponto) :
        custo = MAX
        melhor = None
        for vizinho in self.grafo.getVizinhos(ponto) :
            val = self.avaliaPonto(vizinho)
            if custo > val :
                custo = val
                melhor = vizinho
        return melhor
    
    def criaMovimento (self, atual, proximo) :
        h = self.modelo.getH()
        
        if atual == proximo + 1 : #proximo na esquerda - H -90
            if h == 0 :
                messenger.send("MoveAgente", ["left", 1])
            elif h == 180 :
                messenger.send("MoveAgente", ["right", 3])
            elif h == 90 :
                messenger.send("MoveAgente", ["left", 2])
        elif atual == proximo - 1 : #proximo na direita - H 90
            if h == 0 :
                messenger.send("MoveAgente", ["right", 1])
            elif h == 180 :
                messenger.send("MoveAgente", ["left", 1])
            elif h == -90 :
                messenger.send("MoveAgente", ["right", 2])
        elif atual == proximo + 10 : #proximo acima - H 180
            if h == 0 :
                messenger.send("MoveAgente", ["right", 2])
            elif h == 90 :
                messenger.send("MoveAgente", ["right", 1])
            elif h == -90 :
                messenger.send("MoveAgente", ["right", 3])
        elif atual == proximo - 10 : #proximo abaixo - H 0
            if h == 180 :
                messenger.send("MoveAgente", ["right", 2])
            elif h == 90 :
                messenger.send("MoveAgente", ["right", 1])
            elif h == -90 :
                messenger.send("MoveAgente", ["left", 1])
        messenger.send("MoveAgente", ["forward", 1])
        
    
    def move (self, main, camera):
        self.startPos = self.modelo.getPos()
        h = self.modelo.getH()
        move = False
        #Atualizacao da posicao do personagem
        if (self.keyMap["left"] != 0):
            self.keyMap["left"] = self.keyMap["left"] - 1
            self.modelo.setH(self.modelo.getH() + 90)
            move = True
        if (self.keyMap["right"] != 0):
            self.keyMap["right"] = self.keyMap["right"] - 1
            self.modelo.setH(self.modelo.getH() - 90)
            move = True
        if (self.keyMap["forward"] != 0):
            self.keyMap["forward"] = self.keyMap["forward"] - 1
            self.modelo.setY(self.modelo, - .9)
            move = True
        #Atualizacao da posicao da animacao
        if move == True:
            if self.isMoving is False:
                self.modelo.loop("run")
                self.isMoving = True
        else:
            if self.isMoving:
                self.modelo.stop()
                self.modelo.pose("walk", 5)
                self.isMoving = False
                
        camera.atualizaPosicao()
        
        for e in self.CollisionHandler.getEntries():
            if e.getIntoNode().getName() == "personagemSphere":
                self.modelo.setPos(self.startPos)
            elif e.getIntoNode().getName().find("tampa0") >= 0:
                self.modelo.setPos(self.startPos)

    def setKey(self, key, value):
        self.keyMap[key] += value