from panda3d.core import BitMask32, CollisionHandlerQueue, \
    CollisionSphere, CollisionTraverser, CollisionNode, PandaNode, NodePath

class Camera():
    '''
        Classe que inicializa uma camera e suas colisoes. Alem disso, gerencia
        os movimentos da camera e o tratamento de suas colisoes
    '''
    def __init__(self, personagem):
        self.keyMap = {"cam-left":0, "cam-right":0}
        self.personagem = personagem
        
        self.CollisionSphere = CollisionSphere(0, 0, 0, 1)
        self.CollisionNode = CollisionNode('camSphere')
        self.CollisionNode.addSolid(self.CollisionSphere )
        self.CollisionNode.setFromCollideMask(BitMask32.bit(0))
        self.CollisionNode.setIntoCollideMask(BitMask32.allOff())
        self.CollisionNodePath = base.camera.attachNewNode(self.CollisionNode)
        self.CollisionHandler = CollisionHandlerQueue()

    def setKey(self, key, value):
        self.keyMap[key] = value

    def move(self):
        base.camera.lookAt(self.personagem.modelo)
        if (self.keyMap["cam-left"]!=0):
            base.camera.setX(base.camera, -20 * globalClock.getDt())
        if (self.keyMap["cam-right"]!=0):
            base.camera.setX(base.camera, +20 * globalClock.getDt())
        
        '''
            Tratamento das colisoes. A fila de colisoes e esvaziada
        e analizada, tomando as acoes relativas a cada tipo de colisao.
        ''' 
        entries = []
        for i in range(self.CollisionHandler.getNumEntries()):
            entry = self.CollisionHandler.getEntry(i)
            entries.append(entry)
        entries.sort(lambda x,y: cmp(y.getSurfacePoint(render).getZ(),
                                     x.getSurfacePoint(render).getZ()))
        if (len(entries)>0) and (entries[0].getIntoNode().getName() == "Box02"):
            base.camera.setZ(entries[0].getSurfacePoint(render).getZ()+1.0)
        if (base.camera.getZ() < self.personagem.modelo.getZ() + 2.0):
            base.camera.setZ(self.personagem.modelo.getZ() + 2.0)

        self.personagem.floater.setPos(self.personagem.modelo.getPos())
        self.personagem.floater.setZ(self.personagem.modelo.getZ() + 2.0)
        base.camera.lookAt(self.personagem.floater)
    
    '''
        Atualiza a posicao da camera a partir da posicao do personagem
    '''
    def atualizaPosicao(self):
        camvec = self.personagem.modelo.getPos() - base.camera.getPos()
        camvec.setZ(0)
        camdist = camvec.length()
        camvec.normalize()
        if (camdist > 3.0):
            base.camera.setPos(base.camera.getPos() + camvec*(camdist-3))
            camdist = 3.0
        if (camdist < 1.0):
            base.camera.setPos(base.camera.getPos() - camvec*(1-camdist))
            camdist = 1.0

        base.cTrav.traverse(render)
        
        