from Camera import Camera
from Labirinto import Labirinto
from Jogador import Jogador
from Inimigo import Inimigo
from SetUp import SetUp
from direct.showbase.ShowBase import ShowBase
from panda3d.core import CollisionTraverser, PandaNode, NodePath, Fog
from pandac.PandaModules import Point3
from direct.showbase.DirectObject import DirectObject
from Menu import Menu
from MesaMovel import MesaMovel
from MiniGame import MiniGame
from EstadoMemento import EstadoMemento
import time
from random import randint

class Main(ShowBase, DirectObject):
    '''
        Classe principal do programa. Inicializa o personagem e o cenario. Alem disso,
    se relaciona com a classe "SetUp" para inicializar algumas configuracoes.
    '''

    def __init__(self):
        ShowBase.__init__(self)
        self.rede = None
        self.lastTime = time.clock()
        self.accept("iniciaJogo", self.inicia)
        self.minigames = []
        Menu(self)
        
    def inicia(self):
        self.jogadores = []
        self.estado = None

        self.carregaCenario()
        self.carregaJogadores()

        self.cam = Camera(self.personagem)
        SetUp.configuracaoInicial(self, self.personagem, self.cam);
        self.tarefaMovimentos = taskMgr.add(self.movimentos, "moveTask")
        self.colisoes()
        
        self.accept('MiniGame', self.transicaoMiniGame)
        self.accept("acabouNeblina", self.inicializaMiniGame)
        self.accept("fimMiniGame", self.voltaParaJogoBase)
        self.accept("Sync", self.sync)
        self.accept("bola", self.rede.enviaBola)
        
        self.minigames.append(MesaMovel)

    '''
        Registro dos objetos que podem gerar uma colisao. No caso, o personagem
        e sua camera.
    '''
    
    def carregaCenario(self):
        self.cenario = Labirinto()
        self.labirinto = self.cenario.cenario
        
    def carregaJogadores(self):
        self.personagem = Jogador("ralph", self)
        self.modelo = self.personagem.modelo
        self.personagem.modelo.setX(1)
        self.jogadores.append(self.personagem)

        self.inimigo = Inimigo("ralph", self)
        self.accept("Movimento", self.setMovimento)
        self.modeloinimigo = self.inimigo.modelo
        self.modeloinimigo.setX(0)
        self.jogadores.append(self.inimigo)
    
    def setMovimento(self, key, value):
        self.inimigo.setKey(key, value)
    
    def colisoes(self):
        base.cTrav = CollisionTraverser()
        base.cTrav.addCollider(self.personagem.CollisionNodePath, self.personagem.CollisionHandler)
        base.cTrav.addCollider(self.inimigo.CollisionNodePath, self.inimigo.CollisionHandler)
        base.cTrav.addCollider(self.cam.CollisionNodePath, self.cam.CollisionHandler)

    '''
        Delega ao personagem e a camera as acoes de se movimentar
    '''
    def movimentos(self, task):
        for j in self.jogadores :
            j.move(self, self.cam)
        if self.estado == None :
            self.cam.move()
        t = time.clock()
        if t - self.lastTime > .1 :
            self.lastTime = t
            self.rede.enviaSync(self.personagem.modelo.getPos(), self.personagem.modelo.getHpr())
        return task.cont

    def limpaTudo(self):
        self.labirinto.detachNode()
        SetUp.escondeCaixas()
        
    def restaura(self):
        self.labirinto.reparentTo(render)
        for j in self.jogadores :
            j.modelo.reparentTo(render)
        
    def transicaoMiniGame(self):
        self.estado = EstadoMemento.salvaEstado(self.jogadores)
        self.ignore("MiniGame")
        self.limpaTudo()
        SetUp.iniciaNeblina()
        
    def inicializaMiniGame(self):
        classe = self.minigames[randint(0, len(self.minigames) - 1)]
        minigame = classe(self.jogadores)
        self.accept("bolaPos", minigame.bolaPos)
        minigame.run()
        SetUp.removeNeblina()
        
    def voltaParaJogoBase(self):
        self.ignore("bolaPos")
        self.rede.enviaMsgFim()
        self.accept('MiniGame', self.transicaoMiniGame)
        self.restaura()
        SetUp.reposicionaCaixas()
        EstadoMemento.recuperaEstado(self.jogadores, self.estado)
        self.estado = None
        
    def sync (self, x, y, z, h, p, r):
        self.inimigo.sync(x, y, z, h, p, r)
        
m = Main()
m.run()
