from direct.showbase.DirectObject import DirectObject
from Camera import Camera
import sys
from TelaCria import TelaCria
from TelaProcura import TelaProcura
#Menus
MENU_APRESENTACAO   = 0
MENU_PRINCIPAL      = 1
MENU_OPCOES         = 2
MENU_INICIAR        = 3
MENU_CREDITOS       = 4
#Opcoes do menu de apresentacao
SAIR_APRESENTACAO   = 10
#Opcoes do menu principal    
INICIAR_PRINCIPAL   = 11
OPCOES_PRINCIPAL    = 21
SAIR_PRINCIPAL      = 31
#Opcoes do menu de opcoes
CREDITOS_OPCOES     = 12
SOM_OPCOES          = 22
VOLTAR_OPCOES       = 32
#Opcoes do menu de iniciar partida
CRIAR_INICIAR       = 13
PROCURAR_INICIAR    = 23
VOLTAR_INICIAR      = 33
#Opcoes do menu de creditos
VOLTAR_CREDITOS     = 14

LOCAL = "../modelos/menu/"

pos = {"principalOn": (0,0,0), "principalOff": (-1.9,0,0), 
       "opcoesOn":    (0,0,0), "opcoesOff":    (0,-1.9,0),
       "iniciarOn":   (0,0,0), "iniciarOff":   (0,1.9,0),
       "creditosOn":  (0,0,0), "creditosOff":  (1.9,0,0)}

class Menu(DirectObject) :
    
    def __init__ (self, main) :
        self.main = main
        self.OK = False
        self.cam = base.camera
        base.disableMouse()
        base.camera.setPos(0, -4, 0)
#        base.camera.lookAt(0,0,0.2)

        self.menu     = loader.loadModel(LOCAL+"menuBase")
        self.iniciar  = loader.loadModel(LOCAL+"22iniciar")
        self.opcoes   = loader.loadModel(LOCAL+"23opcoes")
        self.sair     = loader.loadModel(LOCAL+"24sair")
        self.creditos = loader.loadModel(LOCAL+"41creditos")
        self.audio    = loader.loadModel(LOCAL+"42audio")
        self.audioOn  = loader.loadModel(LOCAL+"43ligado")
        self.audioOff = loader.loadModel(LOCAL+"44desligado")
        self.voltarOp = loader.loadModel(LOCAL+"45voltar")
        self.criar    = loader.loadModel(LOCAL+"31criar")
        self.procurar = loader.loadModel(LOCAL+"32procurar")
        self.voltarIn = loader.loadModel(LOCAL+"33voltar")
        self.voltarCr = loader.loadModel(LOCAL+"53voltar")
        
        self.menu.reparentTo(render)
        self.menu.setPos(0, 30, 0)
        self.menu.setHpr(0, 90, 0)
        
        
        #self.menu.setScale(0.9)
        self.iniciar.reparentTo(self.menu)
        self.iniciar.setPos(pos["principalOn"])
        self.opcoes.reparentTo(self.menu)
        self.opcoes.setPos(pos["principalOff"])
        self.sair.reparentTo(self.menu)
        self.sair.setPos(pos["principalOff"])
        self.creditos.reparentTo(self.menu)
        self.creditos.setPos(pos["opcoesOn"])
        self.audio.reparentTo(self.menu)
        self.audio.setPos(pos["opcoesOff"])
        self.audioOn.reparentTo(self.menu)
        self.audioOn.setPos(pos["opcoesOn"])
        self.audioOff.reparentTo(self.menu)
        self.audioOff.setPos(pos["opcoesOff"])
        self.voltarOp.reparentTo(self.menu)
        self.voltarOp.setPos(pos["opcoesOff"])
        self.criar.reparentTo(self.menu)
        self.criar.setPos(pos["iniciarOn"])
        self.procurar.reparentTo(self.menu)
        self.procurar.setPos(pos["iniciarOff"])
        self.voltarIn.reparentTo(self.menu)
        self.voltarIn.setPos(pos["iniciarOff"])
        self.voltarCr.reparentTo(self.menu)
        self.voltarCr.setPos(pos["creditosOn"])
        
        self.menuAtual = MENU_APRESENTACAO
        
#        print self.menu.find('**/22iniciar')
        
        self.opcoesMenu = {
            SAIR_APRESENTACAO   : self.goToPrincipal,
            INICIAR_PRINCIPAL   : self.goToIniciar,
            OPCOES_PRINCIPAL    : self.goToOpcoes,
            SAIR_PRINCIPAL      : self.finaliza,
            CREDITOS_OPCOES     : self.goToCreditos,
            SOM_OPCOES          : self.finaliza,
            VOLTAR_OPCOES       : self.goToPrincipal,
            CRIAR_INICIAR       : self.finaliza,
            PROCURAR_INICIAR    : self.finaliza,
            VOLTAR_INICIAR      : self.goToBackPrincipal,
            VOLTAR_CREDITOS     : self.goToOpcoes,
        }

        self.opcao = 1
        self.somLigado = 1
        self.keyMap = {"up" : 0, "down" : 0, "seleciona" : 0, "left":0, "right":0}
        self.accept("arrow_up",         self.setKey, ["up",        1])
        self.accept("arrow_down",       self.setKey, ["down",      1])
        self.accept("arrow_left",       self.setKey, ["left",      1])
        self.accept("arrow_right",      self.setKey, ["right",     1])
        self.accept("enter",            self.setKey, ["seleciona", 1])
        
        taskMgr.add(self.proxMenu, "altera menu")
        
    'Command Dispatch Pattern'
    def goTo (self, menu) :
        func = self.opcoesMenu[menu]
        func()
        self.opcao = 1
        
    def goToPrincipal (self) :
        self.menu.hprInterval(1, (0, 90, -90)).start()
        self.menuAtual = MENU_PRINCIPAL
        
    def goToOpcoes (self) :
        self.menu.hprInterval(1, (0, 180, -90)).start()
        self.menuAtual = MENU_OPCOES
    
    def goToBackPrincipal (self) :
        self.menu.hprInterval(1, (0, 90, -90)).start()
        self.menuAtual = MENU_PRINCIPAL
    
    def goToIniciar (self) :
        self.menu.hprInterval(1, (0, 0, -90)).start()
        self.menuAtual = MENU_INICIAR
        
    def goToCreditos (self) :
        self.menu.hprInterval(1, (0, 270, -90)).start()
        self.menuAtual = MENU_CREDITOS

    def finaliza (self) :
        taskMgr.remove("altera menu")
        
    def setKey(self, key, value):
        self.keyMap[key] = value
        
    def proxMenu (self, task) :
        if (self.keyMap["left"] != 0):
            self.setKey("left",0)
            if(self.menuAtual == MENU_OPCOES):
                if(self.opcao == 2):
                    self.audioOn.setPos(pos["opcoesOn"])
                    self.somLigado = 1
                    self.audioOff.setPos(pos["opcoesOff"])
        if (self.keyMap["right"] != 0):
            self.setKey("right",0)
            if(self.menuAtual == MENU_OPCOES):
                if(self.opcao == 2):
                    self.audioOn.setPos(pos["opcoesOff"])
                    self.somLigado = 0
                    self.audioOff.setPos(pos["opcoesOn"])
        if (self.keyMap["up"] != 0):
            self.setKey("up", 0)
            if(self.menuAtual == MENU_PRINCIPAL):
                self.opcao = max(self.opcao -1 , 1)
                if(self.opcao == 1):
                    self.opcoes.setPos(pos["principalOff"])
                    self.iniciar.setPos(pos["principalOn"])
                elif (self.opcao == 2):
                    self.sair.setPos(pos["principalOff"])
                    self.opcoes.setPos(pos["principalOn"])
            elif(self.menuAtual == MENU_OPCOES):
                self.opcao = max(self.opcao -1,1)
                if(self.opcao == 1):
                    self.creditos.setPos(pos["opcoesOn"])
                    self.audio.setPos(pos["opcoesOff"])
                elif(self.opcao == 2):
                    self.audio.setPos(pos["opcoesOn"])
                    self.voltarOp.setPos(pos["opcoesOff"])
            elif(self.menuAtual == MENU_INICIAR):
                self.opcao = max(self.opcao-1,1)
                if(self.opcao == 1):
                    self.criar.setPos(pos["iniciarOn"])
                    self.procurar.setPos(pos["iniciarOff"])
                elif(self.opcao == 2):
                    self.procurar.setPos(pos["iniciarOn"])
                    self.voltarIn.setPos(pos["iniciarOff"])
        elif (self.keyMap["down"] != 0):
            self.setKey("down", 0)
            if(self.menuAtual == MENU_PRINCIPAL):
                self.opcao = min(self.opcao +1 , 3)
                if(self.opcao == 2):
                    self.iniciar.setPos(pos["principalOff"])
                    self.opcoes.setPos(pos["principalOn"])
                elif (self.opcao == 3):
                    self.opcoes.setPos(pos["principalOff"])
                    self.sair.setPos(pos["principalOn"])
            elif(self.menuAtual == MENU_OPCOES):
                self.opcao = min(self.opcao + 1, 3)
                if(self.opcao == 2):
                    self.audio.setPos(pos["opcoesOn"])
                    self.creditos.setPos(pos["opcoesOff"])
                elif(self.opcao == 3):
                    self.voltarOp.setPos(pos["opcoesOn"])
                    self.audio.setPos(pos["opcoesOff"])
            elif(self.menuAtual == MENU_INICIAR):
                self.opcao = min(self.opcao+1, 3)
                if(self.opcao == 2):
                    self.procurar.setPos(pos["iniciarOn"])
                    self.criar.setPos(pos["iniciarOff"])
                elif(self.opcao == 3):
                    self.voltarIn.setPos(pos["iniciarOn"])
                    self.procurar.setPos(pos["iniciarOff"])
        elif (self.keyMap["seleciona"] != 0):
            self.setKey("seleciona", 0) 
            if(self.menuAtual == MENU_APRESENTACAO):
                self.goTo(SAIR_APRESENTACAO)
            elif(self.menuAtual == MENU_PRINCIPAL):
                if(self.opcao == 1):
                    self.goTo(INICIAR_PRINCIPAL)
                elif(self.opcao == 2):
                    self.goTo(OPCOES_PRINCIPAL)
                    self.iniciar.setPos(pos["principalOn"])
                    self.opcoes.setPos(pos["principalOff"])
                elif(self.opcao == 3):
                    sys.exit()
            elif(self.menuAtual == MENU_OPCOES):
                if(self.opcao == 1):
                    self.goTo(CREDITOS_OPCOES)
                elif(self.opcao == 2):
                    if(self.somLigado == 1):
                        self.audioOff.setPos(pos["opcoesOn"])
                        self.audioOn.setPos(pos["opcoesOff"])
                        self.somLigado = 0
                    elif(self.somLigado == 0):
                        self.audioOff.setPos(pos["opcoesOff"])
                        self.audioOn.setPos(pos["opcoesOn"])
                        self.somLigado = 1
                elif(self.opcao == 3):
                    self.creditos.setPos(pos["opcoesOn"])
                    self.voltarOp.setPos(pos["opcoesOff"])
                    self.goTo(VOLTAR_INICIAR)
            elif(self.menuAtual == MENU_INICIAR):
                if(self.opcao == 1 and self.main.rede == None):
                    TelaCria(self, self.main)
                elif(self.opcao == 2 and not self.OK):
                    self.OK = True
                    TelaProcura(self, self.main)
                elif(self.opcao == 3):
                    self.criar.setPos(pos["iniciarOn"])
                    self.voltarIn.setPos(pos["iniciarOff"])
                    self.goTo(VOLTAR_INICIAR)
            elif(self.menuAtual == MENU_CREDITOS):
                self.goTo(VOLTAR_CREDITOS)
        return task.cont    