from MiniGame import MiniGame
from panda3d.core import CollisionTraverser,CollisionNode
from panda3d.core import CollisionHandlerQueue,CollisionRay
from panda3d.core import Material,LRotationf,NodePath
from panda3d.core import AmbientLight,DirectionalLight
from panda3d.core import TextNode
from panda3d.core import Vec3,Vec4,BitMask32
from panda3d.core import TransparencyAttrib
from direct.task.Task import Task
import time

ACELERACAO = 70
VELOCIDADE_MAX = 5
VELOCIDADE_MAX_QUADRADO = VELOCIDADE_MAX ** 2

VETOR = Vec3(0,0,1) 

class MesaMovel (MiniGame) :
    
    def posiciona_camera (self) :
        base.camera.setPosHpr(0, 0, 36, 0, -90, 0)
        
    def carrega_modelos (self) :
        self.cenario = self.carregarModelo("minibase")
        self.cenario.reparentTo(render)
       
        self.tampa = self.carregarModelo("minitampa")
        self.tampa.reparentTo(self.cenario)
        self.tampa.setPos(0, 0, 3)
        self.tampa.setTransparency(TransparencyAttrib.MAlpha)
        self.tampa.setAlphaScale(0.4)
        
        self.raizBola = render.attachNewNode("raizBola")
        self.bola = loader.loadModel("../modelos/ball")
        self.bola.setScale(.9)
        self.bola.reparentTo(self.raizBola)
        
        for j in self.jogadores :
            j.modelo.reparentTo(self.tampa)
            j.modelo.setPos(-4.5, -7, j.modelo.getZ() + 1)
            j.modelo.setTransparency(TransparencyAttrib.MNone)
        self.jogadores[1].modelo.setPos(4.5, 7, j.modelo.getZ())
        
    def setup_colisao (self) :
        self.bolaSphere = self.bola.find("**/ball")
        self.bolaSphere.node().setFromCollideMask(BitMask32.bit(0))
        self.bolaSphere.node().setIntoCollideMask(BitMask32.allOff())
        
        self.bolaGroundRay = CollisionRay()
        self.bolaGroundRay.setOrigin(0, 0, 10)
        self.bolaGroundRay.setDirection(0, 0, -1)
        
        self.bolaGroundCol = CollisionNode('groundRay')
        self.bolaGroundCol.addSolid(self.bolaGroundRay)
        self.bolaGroundCol.setFromCollideMask(BitMask32.bit(1))
        self.bolaGroundCol.setIntoCollideMask(BitMask32.allOff())
        
        self.bolaGroundColNp = self.raizBola.attachNewNode(self.bolaGroundCol)
        
        self.cHandler = CollisionHandlerQueue()
        
        base.cTrav.addCollider(self.bolaSphere, self.cHandler)
        base.cTrav.addCollider(self.bolaGroundColNp, self.cHandler)
        
    def setup_luzes (self) :
        luzAmbiente = AmbientLight("ambientLight")
        luzAmbiente.setColor(Vec4(.55, .55, .55, 1))
        luzDirecional = DirectionalLight("directionalLight")
        luzDirecional.setDirection(Vec3(0, 0, -1))
        luzDirecional.setColor(Vec4(0.375, 0.375, 0.375, 1))
        luzDirecional.setSpecularColor(Vec4(1, 1, 1, 1))
        self.raizBola.setLight(render.attachNewNode(luzAmbiente))
        self.raizBola.setLight(render.attachNewNode(luzDirecional))
        
    def carrega (self) :
        self.posiciona_camera()
        self.carrega_modelos()
        self.setup_colisao()
        self.setup_luzes()
        self.lastTime = time.clock()
        
        m = Material()
        m.setSpecular(Vec4(1,1,1,1))
        m.setShininess(96)
        self.bola.setMaterial(m, 1)
        
    def tratamentoColisaoComChao (self, colEntry):
        novoZ = colEntry.getSurfacePoint(render).getZ()
        self.raizBola.setZ(novoZ + .4)
    
        normal = colEntry.getSurfaceNormal(render)
        aceleracaoLateral = normal.cross(VETOR)

        self.aceleracaoV = normal.cross(aceleracaoLateral)

    def tratamentoColisaoComParede (self, colEntry):
        normal = colEntry.getSurfaceNormal(render) * -1 
        velAtual = self.bolaV.length()
        inVec = self.bolaV / velAtual
        velAngular = normal.dot(inVec)
        dir = colEntry.getSurfacePoint(render) - self.raizBola.getPos()
        dir.normalize()
        angulo = normal.dot(dir)

        if velAngular > 0 and angulo > .995:
            vetorReacao = (normal * normal.dot(inVec * -1) * 2) + inVec
        
            self.bolaV = vetorReacao * (velAtual * (((1-velAngular)*.5)+.5))
            
            disp = (colEntry.getSurfacePoint(render) - colEntry.getInteriorPoint(render))
            novaPos = self.raizBola.getPos() + disp

            self.raizBola.setPos(novaPos)

    def bolaPos (self, x, y, z) :
        self.raizBola.setPos(x, y, z)
        
    def inBuraco (self) :
        x = self.raizBola.getX()
        y = self.raizBola.getY()
        if x > 3.6 and x < 4 and y > -6.7 and y < -6.1 :
            return True
        return False
        
    def loop (self, task):
        dt = task.time - task.last
        task.last = task.time

        if dt > .2: return task.cont   

        self.posiciona_camera()
        for i in range(self.cHandler.getNumEntries()):
            entry = self.cHandler.getEntry(i)
            if entry.getIntoNode().getName().find("parede") >= 0 :
                self.tratamentoColisaoComParede(entry)
            elif entry.getIntoNode().getName() == "fundo" :
                self.tratamentoColisaoComChao(entry)
            elif entry.getIntoNode().getName() == "buraco" and self.inBuraco() :
                self.finaliza()
                self.limpa()
                return task.done

        novoP = novoR = 0
        
        for j in self.jogadores :
            novoP += j.modelo.getY()
            novoR += j.modelo.getX()
            
        self.cenario.setP(novoP * -2)
        self.cenario.setR(novoR * 2)
        
        self.bolaV += self.aceleracaoV * dt * ACELERACAO
        
        if self.bolaV.lengthSquared() > VELOCIDADE_MAX_QUADRADO:
            self.bolaV.normalize()
            self.bolaV *= VELOCIDADE_MAX

        self.raizBola.setPos(self.raizBola.getPos() + (self.bolaV * dt))
        t = time.clock()
        if t - self.lastTime > .05 :
            messenger.send("bola", [self.raizBola.getPos()])
            self.lastTime = t

        rotacaoAnterior = LRotationf(self.bola.getQuat())
        eixo = VETOR.cross(self.bolaV)
        novaRotacao = LRotationf(eixo, 45.5 * dt * self.bolaV.length())
        self.bola.setQuat(rotacaoAnterior * novaRotacao)
        
        return task.cont

    def inicia (self):
        posInicial = (-4.5, 7, self.cenario.getZ() + 2.9)
#        posInicial = (3.8, -6.5, self.cenario.getZ() + 2.9)
        self.raizBola.setPos(posInicial)
        self.bolaV = Vec3(0,0,0)
        self.aceleracaoV = Vec3(0,0,0)

        self.mainLoop = taskMgr.add(self.loop, "loop")
        self.mainLoop.last = 0
    
    def finaliza (self):
        taskMgr.remove(self.mainLoop)
        self.posiciona_camera()
        
    def limpa (self) :
        self.ignore("FimDeJogo")
        self.raizBola.remove()
        self.cenario.remove()
        messenger.send('fimMiniGame')