from pandac.PandaModules import Point3
from direct.actor.Actor import Actor
from showbase.DirectObject import DirectObject
from panda3d.core import PandaNode, NodePath, Camera, TextNode, ClockObject
from panda3d.core import CollisionHandlerQueue, \
    CollisionSphere, CollisionTraverser, CollisionNode, PandaNode, NodePath
from task.Task import TaskManager

class Personagem(DirectObject):
    '''
    Abstracao de um personagem e de suas caracteristicas. Essa classe e responsavel
    pelo modelo, pelas colisoes e pela movimentacao de um personagem.
    '''

    def __init__(self, nome, main):
        self.startPos = Point3(0, 0, 0)
        self.modelo = Actor("../modelos/" + nome,
                                 {"run":"../modelos/" + nome + "-run",
                                  "walk":"../modelos/" + nome + "-walk"})
        
        self.main = main
        self.tamanho = 0.3
        self.modelo.reparentTo(render)
        self.modelo.setScale(self.tamanho)
        self.modelo.setPos(self.startPos)

        self.floater = NodePath(PandaNode("floater"))
        self.floater.reparentTo(render)
        self.intervalColisao = None

        self.keyMap = {"left":0, "right":0, "forward":0}

        self.isMoving = False

        self.CollisionSphere = CollisionSphere(0, 0, 3, 1.3)
        self.CollisionNode = CollisionNode('personagemSphere')
        self.CollisionNode.addSolid(self.CollisionSphere)
        self.CollisionNodePath = self.modelo.attachNewNode(self.CollisionNode)
#        self.CollisionNodePath.show()
        self.CollisionHandler = CollisionHandlerQueue()

    def move (self, main, camera):
        self.startPos = self.modelo.getPos()

        #Atualizacao da posicao do personagem
        if (self.keyMap["left"] != 0):
            self.modelo.setH(self.modelo.getH() + 300 * globalClock.getDt())
        if (self.keyMap["right"] != 0):
            self.modelo.setH(self.modelo.getH() - 300 * globalClock.getDt())
        if (self.keyMap["forward"] != 0):
            self.modelo.setY(self.modelo, - 25 * globalClock.getDt())

        #Atualizacao da posicao da animacao
        if (self.keyMap["forward"] != 0) or (self.keyMap["left"] != 0) or (self.keyMap["right"] != 0) :
            if self.isMoving is False:
                self.modelo.loop("run")
                self.isMoving = True
        else:
            if self.isMoving:
                self.modelo.stop()
                self.modelo.pose("walk", 5)
                self.isMoving = False
                
        camera.atualizaPosicao()

        for e in self.CollisionHandler.getEntries():
            if (e.getIntoNode().getName().find("Tube") >= 0 or e.getIntoNode().getName().find("Box") >= 0):
                self.modelo.setPos(self.startPos)
            elif e.getIntoNode().getName() == "personagemSphere":
                self.modelo.setPos(self.startPos)
            elif e.getIntoNode().getName().find("tampa0") >= 0:
                self.modelo.setPos(self.startPos)
            elif e.getIntoNode().getName().find("Interrogacao") >= 0:
                messenger.send('MiniGame')
                self.main.rede.enviaMiniGame()