from direct.gui.OnscreenText import OnscreenText
from math import sin, cos
from panda3d.core import AmbientLight, DirectionalLight, Camera, TextNode, Vec3, \
    Vec4, Fog
from random import seed, randint, random
from showbase.DirectObject import DirectObject
import sys

numeroCaixas = 15
caixas = []

class SetUp():
    tempoInicial = 0
    fog = Fog("Nevoa")
    fog.setColor(1,1,1)
    
    '''
        Classe responsavel por executar comandos de inicializacao de luzes, teclas
        e camera. Tambem posiciona na tela as caixas que servirao de passagem aos
        minigames.
    '''
    
    @staticmethod
    def configuracaoInicial(main, personagem, camera):
        SetUp.__setUpLuzes()
        SetUp.__setUpTeclas(main, personagem, camera)
        SetUp.__setUpCamera(main)
        SetUp.__posicionaCaixas()
    
    @staticmethod
    def removeNeblina():
        render.clearFog()
    
    @staticmethod
    def iniciaNeblina():
        base.setBackgroundColor(1, 1, 1)
        numeroSegundos = 2
        SetUp.tempoInicial = globalClock.getLongTime()
        taskMgr.add(SetUp.neblina, "neblinaTask", extraArgs=[numeroSegundos], appendTask=True)
        
    @staticmethod
    def neblina(numeroSegundos, task):
        tempoFinal = SetUp.tempoInicial + numeroSegundos
        tempoAtual = globalClock.getLongTime()
        
        if(tempoAtual < tempoFinal):
            t = (tempoAtual - SetUp.tempoInicial)/(tempoFinal - SetUp.tempoInicial)
            SetUp.fog.setExpDensity(t)
            render.setFog(SetUp.fog)
        else:
            SetUp.fog.setExpDensity(1)
            render.setFog(SetUp.fog)
            messenger.send("acabouNeblina")
            return task.done
        
        return task.cont
    
    @staticmethod
    def __posicionaCaixas():
        seed()
        for i in range(0, numeroCaixas - 1):
            caixa = loader.loadModel("../modelos/interrogacao")
            caixa.setScale(0.025)
            caixa.setP(90)
            caixa.reparentTo(render)
            caixas.append(caixa)

            raio = randint(1, 10) * 4 + 1.5
            angulo = random() * 360
            x = raio * cos(angulo)
            y = raio * sin(angulo)
            caixa.setPos(x, y, 0.8)
            
        taskMgr.add(SetUp.rotacionaCaixa, "rotaciona")
    
    @staticmethod
    def rotacionaCaixa(task):
        for caixa in caixas:
            caixa.setH((caixa.getH() + 25))
        return task.cont

    @staticmethod
    def escondeCaixas():
        for caixa in caixas:
            caixa.detachNode()

    @staticmethod
    def reposicionaCaixas():
        seed()

        for caixa in caixas:
            raio = randint(1, 10) * 4 + 1.5
            angulo = random() * 360
            x = raio * cos(angulo)
            y = raio * sin(angulo)
            caixa.setPos(x, y, 0.8)
            caixa.reparentTo(render)

    @staticmethod
    def __setUpCamera(main):
        base.disableMouse()
        base.camera.setPos(main.personagem.modelo.getX(), main.personagem.modelo.getY()-20, 5)

    @staticmethod 
    def __setUpLuzes():
        ambientLight = AmbientLight("ambientLight")
        ambientLight.setColor(Vec4(.3, .3, .3, 1))
        directionalLight = DirectionalLight("directionalLight")
        directionalLight.setDirection(Vec3(-5, -5, -5))
        directionalLight.setColor(Vec4(1, 1, 1, 1))
        directionalLight.setSpecularColor(Vec4(1, 1, 1, 1))
        render.setLight(render.attachNewNode(ambientLight))
        render.setLight(render.attachNewNode(directionalLight))
      
    @staticmethod
    def __setUpTeclas(main, personagem, camera):
        main.keyMap = {"cam-left":0, "cam-right":0, "left":0, "right":0, "forward":0, "down":0}
        main.accept("escape", sys.exit)
        main.accept("a", camera.setKey, ["cam-left",1])
        main.accept("s", camera.setKey, ["cam-right",1])
        main.accept("a-up", camera.setKey, ["cam-left",0])
        main.accept("s-up", camera.setKey, ["cam-right",0])

        main.accept("arrow_left", personagem.setKey, ["left",1])
        main.accept("arrow_right", personagem.setKey, ["right",1])
        main.accept("arrow_up", personagem.setKey, ["forward",1])
        main.accept("arrow_left-up", personagem.setKey, ["left",0])
        main.accept("arrow_right-up", personagem.setKey, ["right",0])
        main.accept("arrow_up-up", personagem.setKey, ["forward",0])

    @staticmethod    
    def __setKey(self, key, value):
        self.keyMap[key] = value
