from pandac.PandaModules import *
from direct.task.TaskManagerGlobal import *
from direct.distributed.PyDatagram import PyDatagram
from direct.distributed.PyDatagramIterator import PyDatagramIterator
from rede import *


class Cliente() :
	
	'''
		Classe que implementa opreacoes referentes a um cliente em uma conexao em rede. 
	'''
	
	def __init__ (self, host) :
		#Controle das conexoes
		self.conManager = QueuedConnectionManager()
		self.conReader = QueuedConnectionReader(self.conManager, 0)
		self.conWriter = ConnectionWriter(self.conManager, 0)
		self.myCon = None
		self.host = host
			
	def start (self) :
		tentativas = 0
		while not self.myCon and tentativas < 5:
			self.myCon = self.conManager.openTCPClientConnection(self.host, portAddress, timeout)
			tentativas += 1 
			
		if tentativas == 5 :
			print 'Nao foi possivel se conectar ao host!'
		
		#Conexao estabelecida 
		self.conReader.addConnection(self.myCon)
		taskMgr.add(self.taskReaderPolling, "connection reader", -40)
		
		print 'Conexao estabelecida com sucesso!'
		
	def taskReaderPolling (self, task) :
		# Possui dados para ler
		if self.conReader.dataAvailable() :
			datagram = NetDatagram()
			if self.conReader.getData(datagram) :
				self.processDatagram(datagram)
				
		return task.cont
		
	def processDatagram (self, datagram) :
		iterator = PyDatagramIterator(datagram)
		msgID = iterator.getInt8()
		idEnvio = iterator.getInt8()
		idRecebimento = iterator.getInt8()
		if msgID == ID_MSG :
			self.id = idRecebimento
		elif msgID == MINIGAME_MSG :
			messenger.send("MiniGame")
		elif msgID == POSITION_MSG :
			key = iterator.getString()
			value = iterator.getInt8()
			messenger.send("Movimento", [key, value])
		elif msgID == SYNC_MSG :
			x = iterator.getFloat32()
			y = iterator.getFloat32()
			z = iterator.getFloat32()
			h = iterator.getFloat32()
			p = iterator.getFloat32()
			r = iterator.getFloat32()
			messenger.send("Sync", [x, y, z, h, p, r])
		elif msgID == TRANSICAO_MSG :
			messenger.send("comecou")
			messenger.send("iniciaJogo")
		elif msgID == BOLA_MSG :
			x = iterator.getFloat32()
			y = iterator.getFloat32()
			z = iterator.getFloat32()
			messenger.send("bolaPos", [x, y, z])
		elif msgID == END_MSG :
			messenger.send("FimDeJogo")

	def enviaMsg(self, key, value):
		msg = createMsgPosicao(self.id, key, value)
		self.conWriter.send(msg, self.myCon)
				
	def enviaMsgFim (self):
		msg = createMsgFim(self.id)
		self.conWriter.send(msg, self.myCon)

	def enviaMiniGame (self):
		msg = createMsgMiniGame(self.id)
		self.conWriter.send(msg, self.myCon)
		
	def enviaBola (self, pos) :
		pass
	
	def enviaTransicao(self):
		msg = createMsgTransicao(self.id)
		self.conWriter.send(msg, self.myCon)
		
	def enviaSync(self, pos, hpr):
		msg = createMsgSync(self.id, pos, hpr)
		self.conWriter.send(msg, self.myCon)
