from pandac.PandaModules import *
from direct.task.TaskManagerGlobal import * 
from direct.distributed.PyDatagram import PyDatagram
from direct.distributed.PyDatagramIterator import PyDatagramIterator
from rede import *

class Servidor() :
	'''
		Classe que implementa opreacoes referentes a um servidor em uma conexao em rede. 
	'''
	def __init__(self) :
		#Controle das conexoes
		self.conManager = QueuedConnectionManager()
		
		self.conListener = QueuedConnectionListener(self.conManager, 0)
		self.conReader = QueuedConnectionReader(self.conManager, 0)
		self.conWriter = ConnectionWriter(self.conManager, 0)
		#Conexoes ativas
		self.activeCon = []

	def getHostName(self):
		return self.conManager.getHostName()

	def start (self) :
		self.tcpSocket = self.conManager.openTCPServerRendezvous(portAddress, backlog)
		self.conListener.addConnection(self.tcpSocket)
		
		taskMgr.add(self.taskListenerPolling, "connection listener", -39)
		taskMgr.add(self.taskReaderPolling, "connection reader", -40)
		
	def taskListenerPolling (self, task) :
		# Tentativa de nova conexao
		if self.conListener.newConnectionAvailable() :
			rendezvous = PointerToConnection()
			netAddress = NetAddress()
			newConnection = PointerToConnection()
			# Nova conexao estabelecida
			if self.conListener.getNewConnection(rendezvous, netAddress, newConnection) :
				newConnection = newConnection.p()
				self.activeCon.append(newConnection)
				self.conReader.addConnection(newConnection)
				id = len(self.activeCon)
				msg = self.createMsgID(id)
				self.sendMsg(msg, id)
				messenger.send("guest", [newConnection.getAddress().getIpString()])
				print 'Cliente %s conectou!' % id
		return task.cont
	
	def taskReaderPolling (self, task) :
		# Possui dados para ler
		if self.conReader.dataAvailable() :
			datagram = NetDatagram()
			if self.conReader.getData(datagram) :
				self.processDatagram(datagram)
				
		return task.cont

	def processDatagram (self, datagram) :
		iterator = PyDatagramIterator(datagram)
		msgID = iterator.getInt8()
		idEnvio = iterator.getInt8()
		idRecebimento = iterator.getInt8()
		if msgID == POSITION_MSG :
			key = iterator.getString()
			value = iterator.getInt8()
			messenger.send("Movimento", [key, value])
			for i in range(len(self.activeCon)) :
				if i + 1 != idEnvio :
					client = self.activeCon[i]
					self.conWriter.send(datagram, client)
		elif msgID == SYNC_MSG :
			x = iterator.getFloat32()
			y = iterator.getFloat32()
			z = iterator.getFloat32()
			h = iterator.getFloat32()
			p = iterator.getFloat32()
			r = iterator.getFloat32()
			messenger.send("Sync", [x, y, z, h, p, r])
			for i in range(len(self.activeCon)) :
				if i + 1 != idEnvio :
					client = self.activeCon[i]
					self.conWriter.send(datagram, client)
		elif msgID == MINIGAME_MSG :
			msg = iterator.getString()
			if idRecebimento == BROADCAST_MSG :
				self.broadcastMsg(MINIGAME_MSG, idEnvio, msg)
			messenger.send("MiniGame")
		elif msgID == END_MSG :
			messenger.send("FimDeJogo")
				
	def enviaMsg(self, key, value):
		msg = createMsgPosicao(0, key, value)
		for i in range(len(self.activeCon)) :
			client = self.activeCon[i]
			self.conWriter.send(msg, client)
	
	def enviaMiniGame(self):
		msg = createMsgMiniGame(0)
		for i in range(len(self.activeCon)) :
			client = self.activeCon[i]
			self.conWriter.send(msg, client)
	
	def sendMsg (self, msg, id) :
		client = self.activeCon[id-1]
		self.conWriter.send(msg, client)
		
	def enviaTransicao(self):
		msg = createMsgTransicao(0)
		for i in range(len(self.activeCon)) :
			client = self.activeCon[i]
			self.conWriter.send(msg, client)
			
	def enviaSync(self, pos, hpr):
		msg = createMsgSync(0, pos, hpr)
		for i in range(len(self.activeCon)) :
			client = self.activeCon[i]
			self.conWriter.send(msg, client)
			
	def broadcastMsg (self, tipo, idEnvio, msg) :
		datagram = PyDatagram()
		datagram.addInt8(tipo)
		datagram.addInt8(idEnvio)
		datagram.addInt8(-1)
		datagram.addString(msg)
		for client in self.activeCon :
			self.conWriter.send(datagram, client)
			
	def enviaMsgFim (self):
		msg = createMsgFim(0)
		for client in self.activeCon :
			self.conWriter.send(msg, client)
		
	def enviaBola (self, pos) :
		msg = createMsgBola(0, pos)
		for i in range(len(self.activeCon)) :
			client = self.activeCon[i]
			self.conWriter.send(msg, client)
			
	def createMsgID (self, id) :
		datagram = PyDatagram()
		datagram.addInt8(ID_MSG)
		datagram.addInt8(0)
		datagram.addInt8(id)
		return datagram
			
	def finish (self) :
		for client in self.activeCon :
			self.conReader.removeConnection(client)
		self.activeCon = []
		
		self.conManager.closeConnection(self.tcpSocket)
